#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# get_ip.py
"""
==============================================
get_ip - Get spam sender ip from spip database
==============================================

Description
-----------
This script will get information from SPIP database about all
removed (off) messages. By using IP and hostip.info service,
it will get related country.

Two files are output, the first one, full_ip_list.csv, contain
full data to be open in a spreadsheet and studied. The second,
ip_black_list.txt, is the result after country filtering and
formated to be used by banip SPIP plugin.

External documentations
-----------------------
- http://intrw.net/spip.php?article107
- http://www.hostip.info/use.html


"""
__author__ = u"Benoît LAURENT <benoit@intrw.net>"
__version__ = "0.3"
__date__ = "10 november 2008"

import sys
import time
import urllib2

import MySQLdb

from ConfigParser import RawConfigParser
from IPy import IP # http://software.inl.fr/trac/trac.cgi/wiki/IPy

# db connect informations have to be stored in an ini file 
# in same directoy as script as follow:
# ; file connect.ini
# [DB]
# host=127.0.0.1
# user=root
# passwd=mypass
# db=spip
#

# You can edit following variable to fit your needs.
short_list_file = 'ip_black_list.txt'
full_list_file = 'full_ip_list.csv'
countries = ('(Unknown Country?) (XX)', 'HONG KONG (HK)', 'CHINA (CN)', 
            'JAPAN (JP)', 'UKRAINE (UA)', 'MEXICO (MX)', 'INDIA (IN)')

# Get db connection info and conect to it
connect_info = RawConfigParser()
try:
    connect_info.readfp(open(r'connect.ini', 'r'))
except IOError:
    print('db connect informations have to be stored in an ini'
        ' file in same directoy as script (see source)')
    sys.exit(1)
try:
    db=MySQLdb.connect(host=connect_info.get('DB', 'host'),
                   user=connect_info.get('DB', 'user'), 
                   passwd=connect_info.get('DB', 'passwd'), 
                   db=connect_info.get('DB', 'db'))
except StandardError, e:
    print("Unable to connect to MySQL, see error:")
    print(e)
    sys.exit(2)

# Open output files
try:
    short_list = open(short_list_file, 'w')
    full_list = open(full_list_file, 'w')
except IOError:
    print('Unable to open output files.')
    sys.exit(3)

# Get IPs, mails, name and number of removed messages per IP from db
c=db.cursor()
c.execute("SELECT `auteur`, `email_auteur`, `ip`, count(`ip`) " 
          "FROM `spip_forum` WHERE `statut` LIKE 'off' "
          "GROUP BY `ip` order by count(`ip`) desc")

# For each IP get the country and write data to file.
ips = []

for auteur, email, ip, cmpt in c.fetchall():
    try:
        response = urllib2.urlopen('http://api.hostip.info/get_html.php?ip=%s' % ip)
    except IOError, e:
        print('Unable to get informatio from Internet, see error:')
        print(e)
        sys.exit(4)

    country = response.readline().replace('Country: ', '').rstrip()
    print(" %s\t%s\t%s\t%s\t%s" % (auteur, email, ip, cmpt, country))
    if country in countries:
        ips.append(IP(ip).make_net('255.255.255.0'))
    full_list.write("%s,%s,%s,%s,%s\n" % (auteur, email, ip, cmpt, country))
    time.sleep(1)

sort_ip = list(set(ips))
sort_ip.sort()
short_ip = ["%s\n" % ip.strNormal(0) for ip in sort_ip]
short_list.writelines(short_ip)

short_list.close()
full_list.close()
